package com.yeejoin.amos.fas.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.fas.business.action.model.ContingencyRo;
import com.yeejoin.amos.fas.business.action.mq.WebMqttComponent;
import com.yeejoin.amos.fas.business.constants.FasConstant;
import com.yeejoin.amos.fas.business.dao.mapper.*;
import com.yeejoin.amos.fas.business.dao.repository.*;
import com.yeejoin.amos.fas.business.feign.JcsFeign;
import com.yeejoin.amos.fas.business.feign.PrivilegeFeign;
import com.yeejoin.amos.fas.business.feign.RemoteSecurityService;
import com.yeejoin.amos.fas.business.service.intfc.IDictService;
import com.yeejoin.amos.fas.business.service.intfc.IEmergencyTaskService;
import com.yeejoin.amos.fas.business.service.intfc.IPlanStepService;
import com.yeejoin.amos.fas.business.service.intfc.IPlanVisual3dService;
import com.yeejoin.amos.fas.business.service.model.ToipResponse;
import com.yeejoin.amos.fas.business.util.JSONUtil;
import com.yeejoin.amos.fas.business.util.StringUtil;
import com.yeejoin.amos.fas.business.vo.*;
import com.yeejoin.amos.fas.common.enums.PlanReplyMessageEnum;
import com.yeejoin.amos.fas.core.util.CommonResponse;
import com.yeejoin.amos.fas.core.util.CommonResponseUtil;
import com.yeejoin.amos.fas.core.util.ResponseModel;
import com.yeejoin.amos.fas.dao.dto.PlanStepJsonVO;
import com.yeejoin.amos.fas.dao.entity.*;
import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.feign.privilege.model.RoleModel;
import com.yeejoin.amos.feign.systemctl.Systemctl;
import com.yeejoin.amos.feign.systemctl.model.MessageModel;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service("planVisual3dService")
public class PlanVisual3dServiceImpl implements IPlanVisual3dService {

    @Autowired
    ITextPlanDao iTextPlanDao;

    @Autowired
    PlanVisual3dMapper planVisual3dMapper;

    @Autowired
    IDictService dictService;

    @Autowired
    private DictMapper dictMapper;

    @Autowired
    PlanOperationRecordMapper planOperationRecordMapper;

    @Autowired
    IContingencyOriginalDataDao contingencyOriginalDataDao;

    @Autowired
    private IContingencyPlanInstanceRepository contingencyPlanInstanceRepository;

    @Autowired
    private IPlanRuleDao planRuleDao;

    @Autowired
    private IPlanOperationRecordDao planOperationRecordDao;

    @Autowired
    private IPlanDetailDao planDetailDao;

    @Autowired
    private ContingencyInstanceInfoMapper contingencyInstanceInfoMapper;

    @Autowired
    IContingencyPlanInstanceRepository iContingencyPlanInstanceRepository;

    @Autowired
    private ContingencyPlanInstanceMapper contingencyPlanInstanceMapper;

    @Autowired
    private WebMqttComponent webMqttComponent;

    @Autowired
    private IPlanStepService planStepService;

    @Autowired
    private IContingencyPlanInstanceRepository repository;

    @Autowired
    private RedisTemplate redisTemplate;

    @Value("${spring.application.name}")
    private String serviceName;

    @Value("${station.name}")
    private String stationName;

    @Autowired
    private PrivilegeFeign privilegeFeign;

    @Autowired
    private RemoteSecurityService remoteSecurityService;

    @Value("${plan.instance.personImg}")
    private String personImg;

    @Value("${plan.instance.playImg}")
    private String playImg;

    @Value("${aurora.push.switch}")
    private Boolean auroraPushSwitch;

    @Autowired
    JcsFeign jcsFeign;

    @Lazy
    @Autowired
    private IEmergencyTaskService emergencyTaskService;
    @Autowired
    private EmergencyTaskRoleMapper emergencyTaskRoleMapper;

    @Override
    public void uploadTextPlan(String appId, Map<String, String> pathNameMap) {
        if (null != pathNameMap && !pathNameMap.isEmpty()) {
            for (Map.Entry<String, String> entry : pathNameMap.entrySet()) {
                TextPlan textPlan = new TextPlan();
                textPlan.setAppId(appId);
                textPlan.setTextName(entry.getValue());
                textPlan.setFilePath(entry.getKey());
                iTextPlanDao.save(textPlan);
            }
        }
    }

    @Override
    public TextPlan getTextPlanInfoById(Long id) {
        Optional<TextPlan> textPlan = iTextPlanDao.findById(id);
        if (textPlan.isPresent()) {
            return textPlan.get();
        }
        return null;
    }

    @Override
    public CommonResponse getResourceListByType(String type) {
        //1.校验是否是约定的字典，不存在则提示失败
        Dict dict = new Dict();
        dict.setDictCode(FasConstant.PLAN_SOURCE_TYPE);
        dict.setDictValue(type);
        List<Dict> dictList = dictService.getDictList(dict);
        if (CollectionUtils.isEmpty(dictList)) {
            return CommonResponseUtil.failure(type + "字典类型不存在");
        }
        //2.返回存在的数据
//        return CommonResponseUtil.success(planVisual3dMapper.getResourceListByType(type));
        return CommonResponseUtil.success();
    }

    @Override
    public List<Map<String, Object>> getResourceById(String type, Long id) {
        //1.查询出要返回数据
//        Map<String, Object> map = planVisual3dMapper.queryOneByTypeAndId(type, id);
        Map<String, Object> map = new HashMap<String, Object>();
        //1.1无数据则返回
        if (CollectionUtils.isEmpty(map)) {
            return new ArrayList<>();
        }
        //2.加工数据
        //2.0查询出中英文对照字典映射
        Dict dict = new Dict();
        dict.setDictCode(type);
        List<Dict> dictList = dictService.getDictList(dict);
        Map<String, String> dicts = dictList.stream().collect(Collectors.toMap(Dict::getDictValue, Dict::getDictName, (key1, key2) -> key2));

        //2.1换key为中文
        Map<String, Object> tempMap = new HashMap<String, Object>();
        map.forEach((k, v) -> {
            if (dicts.containsKey(k)) {
                tempMap.put(dicts.get(k), v);
            }
        });

        //2.2map转list
        List<Map<String, Object>> list = new ArrayList<>();
        list = tempMap.entrySet().stream().map(e -> {
            Map<String, Object> newMap = new HashMap<String, Object>();
            newMap.put("label", e.getKey());
            newMap.put("value", e.getValue());
            return newMap;
        }).collect(Collectors.toList());
        return list;
    }

    @Override
    public List<Map<String, Object>> getResourceTypeList() {
        Dict dict = new Dict();
        dict.setDictCode(FasConstant.PLAN_SOURCE_TYPE);
        List<Dict> dictList = dictMapper.getDictList(dict);
        List<Map<String, Object>> list = new ArrayList<>();
        list = dictList.stream().map(e -> {
            Map<String, Object> newMap = new HashMap<String, Object>();
            newMap.put("key", e.getDictValue());
            newMap.put("value", e.getDictValue());//前端级联Cascader使用
            newMap.put("isLeaf", false);////前端级联Cascader使用
            newMap.put("dataType", "type");
            newMap.put("label", e.getDictName());
            return newMap;
        }).collect(Collectors.toList());
        return list;
    }

    @Override
    public List<TreeSubjectVo> getTextPlanBySubjectId(String appId) {
        //关联文字预案
        List<TreeSubjectVo> subjectVos = new ArrayList<>();
        List<TextPlan> textPlans = iTextPlanDao.findByAppId(appId);
        textPlans.forEach(t -> {
            TreeSubjectVo textTreeSubjectVo = new TreeSubjectVo();
            textTreeSubjectVo.setParentId(String.valueOf(t.getAppId()));
            textTreeSubjectVo.setTreeName(t.getTextName());
            textTreeSubjectVo.setId(String.valueOf(t.getId()));
            textTreeSubjectVo.setType("textNode");
            subjectVos.add(textTreeSubjectVo);
        });
        return subjectVos;
    }

    @Override
    public void deleteTextPlanFile(Long id) {
        iTextPlanDao.deleteById(id);
    }

    @Override
    public String getNewestBatchNo() {
        return planOperationRecordMapper.getNewestBatchNo();
    }

    @Override
    public ToipResponse getPlanStepByBatchNo(String batchNo) {
        ToipResponse toipResponse = new ToipResponse();
        String planStep = "";
        if (redisTemplate.hasKey("planStep")) {
            planStep = Objects.requireNonNull(redisTemplate.opsForValue().get("planStep")).toString();
        } else {
            planStep = planStepService.getPlanStep();
        }
        JSONArray objects = JSON.parseArray(planStep);
        Map<String, Object> msgContext = new HashMap<>(2);
        if (StringUtil.isNotEmpty(planStep) && StringUtils.isNotBlank(batchNo)) {
            // 查询当前步骤
            ContingencyOriginalData contingencyOriginalData = contingencyOriginalDataDao.findByBatchNo(batchNo);
            if( null == contingencyOriginalData) {
                return null;
            }
            msgContext.put("currentStep", contingencyOriginalData.getStep());
            msgContext.put("confirm", contingencyOriginalData.getConfirm());
            msgContext.put("createDate", contingencyOriginalData.getCreateDate());
            String caseId = contingencyPlanInstanceRepository.findByBatchNoAndRecordType(batchNo, "OPERATE");
            if (StringUtils.isNotEmpty(caseId)) {
                msgContext.put("caseId", caseId);
            }
            toipResponse.setMsgType("steparea");
            toipResponse.setContingency(contingencyOriginalData);
        }
        msgContext.put("step", objects);
        toipResponse.setMsgContext(msgContext);
        return toipResponse;
    }

    /**
     * 根据电力装备Code获取
     *
     * @param equipCode 电力装备Code
     * @return 预案步骤
     */
    @Override
    public JSONArray getPlanStepByEquipCode(String equipCode) {
        Map<String, String> planStepMap = emergencyTaskRoleMapper.queryPlanStepByEquipCode(equipCode);
        String planStep = planStepMap.get("data");
        return JSON.parseArray(planStep);
    }

    @Override
    public List<PlanStepJsonVO> getPlanStepList(String batchNo) {
        String json = planStepService.getPlanStep();
        List<PlanStepJsonVO> res = JSONObject.parseArray(json, PlanStepJsonVO.class);
        return res;
    }

    @Override
    public List<PlanStepJsonVO> updatePlanStepData(String stepCode, String equipCode, List<String> videos) {
        if (!org.springframework.util.StringUtils.hasText(equipCode)) {
            throw new RuntimeException("设备编码不能为空");
        }
        Map<String, String> planStepMap = emergencyTaskRoleMapper.queryPlanStepByEquipCode(equipCode);
        String planStep = planStepMap.get("data");
        String  planType = planStepMap.get("type");
        List<PlanStepJsonVO> planStepJsonVOS = JSON.parseArray(planStep, PlanStepJsonVO.class);
        for (PlanStepJsonVO planStepJsonVO : planStepJsonVOS) {
            if (planStepJsonVO.getStepCode().equals(stepCode)) {
                ArrayList<Map<String, String>> videoList = new ArrayList<>();
                Map<String, List<Map<String, String>>> videosMap = planStepJsonVO.getVideos();
                videos.forEach(t -> {
                    Map<String, String> hashMap = new HashMap<>();
                    hashMap.put("code", t.split("@")[0]);
                    hashMap.put("name", t.split("@")[1]);
                    videoList.add(hashMap);
                });
                videosMap.put(equipCode, videoList);
                planStepJsonVO.setVideos(videosMap);
            }
        }
        String planStepJson = JSON.toJSONString(planStepJsonVOS);
        planStepService.updatePlanStepByPlanType(equipCode, planStepJson);
        if (redisTemplate.hasKey("planType") && Objects.equals(redisTemplate.boundValueOps("planType").get(0, -1), planType)) {
            redisTemplate.opsForValue().set("planStep", planStepJson);
        }
        return planStepJsonVOS;
    }

    @Override
    public ToipResponse getPlaneRecordByBatchNo(String batchNo) {
        // 根据批次号获取预案记录
        List<Map<String, Object>> instancesList = contingencyPlanInstanceRepository.queryRecord(batchNo, "MESSAGE");
        ToipResponse toipResponse = new ToipResponse();
        if (!ObjectUtils.isEmpty(instancesList)) {
            Map<String, Object> msgContext = new HashMap<>(1);
            msgContext.put("content", instancesList);
            ContingencyOriginalData contingencyOriginalData = contingencyOriginalDataDao.findByBatchNo(batchNo);
            ContingencyInstanceInfoVO infoVO = contingencyInstanceInfoMapper.selectDisposalDetails(batchNo);
            if (!ObjectUtils.isEmpty(infoVO.getPosition())) {
                contingencyOriginalData.setPosition(infoVO.getPosition());
            }
            toipResponse.setMockData(judgeIfPlanDataMock(batchNo));
            toipResponse.setMsgType("recordarea");
            toipResponse.setMsgContext(msgContext);
            toipResponse.setContingency(contingencyOriginalData);
        }
        return toipResponse;
    }

    public boolean judgeIfPlanDataMock(String batchNo) {
        if (StringUtil.isNotEmpty(batchNo)) {
            PlanOperationRecord planOperationRecord = planOperationRecordDao.findByBatchNo(batchNo);
            return planOperationRecord.getPlanPattern() == 4 ? true : false;
        } else {
            int planPattern = getLastPlanPattern();
            return planPattern == 4 ? true : false;
        }
    }

    @Override
    public ToipResponse getPlaneRecordOneByBatchNo(String batchNo) {
        // 根据批次号获取预案记录
        String instances = contingencyPlanInstanceRepository.queryRecordOne(batchNo, "HELPAREA");
        ToipResponse toipResponse = new ToipResponse();
        if (StringUtil.isNotEmpty(instances)) {
            Map<String, Object> stringObjectMap = JSONUtil.toMap(instances);
            ContingencyOriginalData contingencyOriginalData = contingencyOriginalDataDao.findByBatchNo(batchNo);
            toipResponse.setMsgType("helparea");
            toipResponse.setMsgContext(stringObjectMap);
            toipResponse.setContingency(contingencyOriginalData);
        }
        return toipResponse;
    }

    @Override
    @Transactional
    public PlanStepVo updatePlanStep(PlanStepVo planStepVo) {
        // 运行中的设置为重置
        List<PlanOperationRecord> planList = planOperationRecordDao.findByStatus(0);
        if (!CollectionUtils.isEmpty(planList)) {
            // 根据批次号查询预案步骤
            PlanRule planRule = planOperationRecordMapper.getPlanRuleByBatchNo(planStepVo.getBatchNo());
            planRule.setPlanStep(planStepVo.getPlanStep());
            planRuleDao.save(planRule);
        }
        return planStepVo;
    }

//    @Override
//    public void initPlanStep() {
//        redisTemplate.opsForValue().set("planStep", planStepService.getPlanStep());
//        redisTemplate.delete("isMock");
//    }

    @Override
    @Transactional
    public void updatePlanStep(String batchNo, String stepCode, String buttonJson, String contingencyPlanId, String buttonCode, String isExecute) {
        if (StringUtils.isNotBlank(batchNo) && StringUtils.isNotBlank(contingencyPlanId) && StringUtils.isNotBlank(buttonCode)) {
            // 根据批次号查询预案步骤
            PlanRule planRule = planOperationRecordMapper.getPlanRuleByBatchNo(batchNo);
            if (planRule != null) {
                List<PlanStepJsonVO> list = JSONObject.parseArray(planRule.getPlanStep(), PlanStepJsonVO.class);
                List<PlanStepJsonVO> collect = list.stream().peek(x -> {
                    if ("true".equals(isExecute)) {
                        //  更新进度条标志位
                        if (StringUtils.isBlank(x.getTime()) && Integer.parseInt(x.getStepCode()) <= Integer.parseInt(stepCode)) {
                            x.setShowLine(true);
                        }
                    }
                }).collect(Collectors.toList());
                List<PlanStepJsonVO> collect1 = collect.stream().peek(x -> {
                    String code = x.getStepCode();
                    if (code.equals(stepCode)) {
                        if ("true".equals(isExecute)) {
                            if (buttonCode.endsWith("_NOT")) {
                                x.setCheckYesOrNo("0");
                            } else {
                                x.setCheckYesOrNo("1");
                                if (StringUtils.isNotBlank(x.getTime())) {
                                    x.setShowLine(true);
                                }
                            }
                            x.setTime(DateFormatUtils.format(new Date(), "HH:mm:ss"));
                        }
                        //  通过contingencyPlanId获取buttonJson数据
                        x.setBatchNo(batchNo);
                        x.setCaseId(contingencyPlanId);
//                        ContingencyPlanInstance planInstance = iContingencyPlanInstanceRepository.getOne(contingencyPlanId);
//                        x.setButtonJson(planInstance.getContent());
                        x.setButtonJson(buttonJson);
                    }
                }).collect(Collectors.toList());
                planRule.setPlanStep(JSON.toJSONString(collect1));
                planRuleDao.save(planRule);
            }
        }
    }

    @Override
    public void updatePlanStepToRedis(String batchNo, String stepCode, String buttonJson, String contingencyPlanId, String buttonCode, String isExecute, String isAuto) {
        if (StringUtils.isNotBlank(batchNo) && StringUtils.isNotBlank(contingencyPlanId) && StringUtils.isNotBlank(buttonCode)) {
            // 根据批次号查询预案步骤
            String planStep = Objects.requireNonNull(redisTemplate.opsForValue().get("planStep")).toString();
            if (StringUtils.isNotBlank(planStep)) {
                List<PlanStepJsonVO> list = JSONObject.parseArray(planStep, PlanStepJsonVO.class);
                List<PlanStepJsonVO> collect = list.stream().peek(x -> {
                    if ("true".equals(isExecute)) {
                        //  更新进度条标志位
                        // 根据stepCode查找对应的索引index
                        int index = planStepService.getPlanStepIndexByStepCode(stepCode);
                        if (StringUtils.isBlank(x.getTime()) && index > -1 && x.getIndex() <= index) {
                            x.setShowLine(true);
                        }
                    }
                }).collect(Collectors.toList());
                List<PlanStepJsonVO> collect1 = collect.stream().peek(x -> {
                    String code = x.getStepCode();
                    if (code.equals(stepCode)) {
                        if ("true".equals(isExecute)) {
                            if (buttonCode.endsWith("_NOT")) {
                                x.setCheckYesOrNo("0");
                            } else {
                                x.setCheckYesOrNo("1");
                                if (StringUtils.isNotBlank(x.getTime())) {
                                    x.setShowLine(true);
                                }
                                if (StringUtils.isNotBlank(isAuto) && "true".equalsIgnoreCase(isAuto)) {
                                    x.setIsAuto(1);
                                }
                            }
                            x.setTime(DateFormatUtils.format(new Date(), "HH:mm:ss"));
                        }
                        x.setStepStatus(x.getCheckYesOrNo());
                        x.setBatchNo(batchNo);
                        x.setCaseId(contingencyPlanId);
                        x.setButtonJson(buttonJson);
                    }
                }).collect(Collectors.toList());
                redisTemplate.opsForValue().set("planStep", JSON.toJSONString(collect1));
                if(stepCode.equals("0")) {
                    PlanStepJsonVO planStepJsonVO = collect1.get(0);
                    if(!StringUtils.isEmpty(planStepJsonVO.getButtonJson())) {
                        String topic = String.format("/%s/%s/%s", serviceName, stationName, "plan");
                        Map<String, Object> map = new HashMap<>();
                        JSONObject msgContext = new JSONObject();
                        msgContext.put("tips", "alertAlarm");
                        map.put("msgContext", msgContext);
                        map.put("msgType", "alertAlarm");
                        map.put("step", "0");
                        map.put("batchNo", planStepJsonVO.getBatchNo());
                        webMqttComponent.publish(topic, JSON.toJSONString(map));
                    }
                }
            }
        }
    }

    @Override
    public Page<ContingencyPlanInstanceVO> getPlaneRecordByNew(Integer pageNum, Integer size, String batchNo, AgencyUserModel user) {
        Integer planInstanceCount = contingencyPlanInstanceMapper.getPlanInstanceCount(batchNo);
        // 根据批次号获取预案记录
        List<ContingencyPlanInstance> planInstanceListByPage = contingencyPlanInstanceMapper.getPlanInstanceListByPage(batchNo, (pageNum - 1) * size, size);
        List<ContingencyPlanInstanceVO> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(planInstanceListByPage)) {
            planInstanceListByPage.stream().forEach(item -> {
                ContingencyPlanInstanceVO instanceVO = new ContingencyPlanInstanceVO();
                BeanUtils.copyProperties(item, instanceVO);
                instanceVO.setLoginUserId(user.getUserId());
                instanceVO.setPlayImg(playImg);
                list.add(instanceVO);
            });
        }
        List<ContingencyPlanInstanceVO> collect = list.stream().sorted(Comparator.comparing(ContingencyPlanInstanceVO::getCreateDate)).collect(Collectors.toList());
        Page<ContingencyPlanInstanceVO> page = new Page<>();
        page.setTotal(planInstanceCount);
        page.setCurrent(pageNum);
        page.setSize(size);
        page.setRecords(collect);
        return page;
    }

    @Override
    public ContingencyPlanInstance replyMessage(AgencyUserModel user, ContingencyPlanInstance dto) {
//        String json = planStepService.getPlanStep();
//        List<PlanStepJsonVO> result = JSONObject.parseArray(json, PlanStepJsonVO.class);
//        List<String> roleCode = result.stream().map(PlanStepJsonVO::getRoleCode).distinct().collect(Collectors.toList());

        String roleCodes = emergencyTaskService.getRolesByUserId(user.getUserId());
        String roleNames = emergencyTaskService.getRolesNameByUserId(user.getUserId());
        List<String> roleModelList = Arrays.asList(roleCodes.split(","));
        List<String> roleNameList = Arrays.asList(roleNames.split(","));

//        List<RoleModel> roleModelList = new ArrayList<>();
//        for (Long key : user.getOrgRoles().keySet()) {
//            roleModelList.addAll(user.getOrgRoles().get(key));
//        }
//        List<String> roleCodeList = new ArrayList<>();
//        List<String> roleNameList = new ArrayList<>();
//        List<String> realRoleCode = roleModelList.stream().map(RoleModel::getRoleName).distinct().collect(Collectors.toList());
//        realRoleCode.forEach(item -> {
//            if (roleCode.contains(item)) {
//                roleCodeList.add(item);
//            }
//        });
//        roleModelList.forEach(item -> {
//            if (!roleNameList.contains(item.getRoleDesc()) && roleCode.contains(item.getRoleName())) {
//                roleNameList.add(item.getRoleDesc());
//            }
//        });
        dto.setRoleCode(String.join("-", roleModelList));
        dto.setRoleName(String.join("-", roleNameList));
        dto.setStartUserId(user.getUserId());
        dto.setStartUserName(user.getRealName());
        dto.setRecordType("REPLYMESSAGE");
        dto.setCreateDate(new Date());
        ResponseModel responseModel = null;
        try {
            responseModel = jcsFeign.getPersonImage(user.getUserId());
        } catch (Exception e) {
            e.printStackTrace();
        }
        String img = "";
        if (responseModel != null && "SUCCESS".equals(responseModel.getDevMessage())) {
            String resStr = JSON.toJSONString(responseModel.getResult()).replace("\"","");
            img = "null".equalsIgnoreCase(resStr) ? null : img;
        }
        img = StringUtils.isEmpty(img) ? personImg : img;
//        Map<String, String> userInfo = contingencyPlanInstanceMapper.getUserByUserId(user.getUserId());
//        if (!ObjectUtils.isEmpty(userInfo)) {
//            String img = userInfo.get("personImg");
//            dto.setPersonImg(StringUtils.isBlank(img) ? personImg : img);
//        } else {
//            dto.setPersonImg(personImg);
//        }
        dto.setPersonImg(img);
        dto.setContent(StringUtil.isNotEmpty(dto.getContent()) ? dto.getContent() : "");
        ContingencyPlanInstance instance = repository.save(dto);
        if (!ObjectUtils.isEmpty(instance)) {
            String topic = String.format("/%s/%s/%s", serviceName, stationName, "plan");
            Map<String, Object> map = new HashMap<>();
            map.put("contingency", new ContingencyRo());
            map.put("msgContext", Collections.EMPTY_MAP);
            map.put("msgType", "replayMessage");
            webMqttComponent.publish(topic, JSON.toJSONString(map));
        }
        return dto;
    }

    @Override
    public Page<ContingencyPlanInstanceVO> selectDisposalActionPage(int current, int size, String disposalId, List<RoleModel> roleModelList, int dataType) {
        List<String> roles = new ArrayList<>();
        String type = "OPERATE";
        String status = "";
        Page<ContingencyPlanInstanceVO> page = new Page<>(current, size);
        if ("1".equals(String.valueOf(dataType))) {
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
            // 根据当前用户预案角色、未执行动作
            status = "1";
        } else if ("2".equals(String.valueOf(dataType))) {
            // 当前用户预案角色、全部动作数据
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
        }
        int total = 0;
        if (ObjectUtils.isEmpty(status)) {
            total = contingencyInstanceInfoMapper.selectCountDisposalActionPage(type, null, roles, disposalId);
        } else {
            total = contingencyInstanceInfoMapper.selectCountDisposalActionPage(type, status, roles, disposalId);

        }
        long start = (page.getCurrent() - 1) * page.getSize();
        if (total == 0) {
            page.setCurrent(1);
        } else {
            if (total < start) {
                page.setCurrent(1);
                start = 0;
            }
            List<ContingencyPlanInstanceVO> list = contingencyInstanceInfoMapper.selectDisposalActionPage((int) start, size, type, status, roles, disposalId);
            page.setRecords(list);
            page.setTotal(total);
        }
        return page;
    }

    @Override
    public Page<ContingencyPlanInstanceVO> selectTaskActionPage(int current, int size, String disposalId, List<String> roleModelList, int dataType) {
        String type = "TASKOPERATE";
        String status = "";
        Page<ContingencyPlanInstanceVO> page = new Page<>(current, size);
//        for (RoleModel roleModel : roleModelList) {
//            roles.add(roleModel.getRoleName());
//        }
        if (null != roleModelList && roleModelList.size() == 0) {
            return new Page<>();
        }
        if ("1".equals(String.valueOf(dataType))) {
            // 根据当前用户预案角色、未执行动作 (dataType = 2 查询所有)
            status = "1";
        }
        int total = 0;
        if (ObjectUtils.isEmpty(status)) {
            total = contingencyInstanceInfoMapper.selectCountTaskActionPage(type, null, roleModelList, disposalId);
        } else {
            total = contingencyInstanceInfoMapper.selectCountTaskActionPage(type, status, roleModelList, disposalId);

        }
        long start = (page.getCurrent() - 1) * page.getSize();
        if (total == 0) {
            page.setCurrent(1);
        } else {
            if (total < start) {
                page.setCurrent(1);
                start = 0;
            }
            List<ContingencyPlanInstanceVO> list = contingencyInstanceInfoMapper.selectTaskActionPage((int) start, size, type, status, roleModelList, disposalId);
            page.setRecords(list);
            page.setTotal(total);
        }
        return page;
    }

    @Override
    public List<ContingencyPlanInstanceVO> getTaskActionList(String batchNo, int dataType) {
        String type = "TASKOPERATE";
        String status = "";
        if ("1".equals(String.valueOf(dataType))) {
            // 根据当前用户预案角色、未执行动作 (dataType = 2 查询所有)
            status = "1";
        }
        return contingencyInstanceInfoMapper.getTaskActionList(type, status, batchNo);
    }

    @Override
    public List<ContingencyPlanInstanceVO> selectTaskActionList(String batchNo, List<String> roleModelList, int dataType) {
        String type = "TASKOPERATE";
        String status = "";
        if ("1".equals(String.valueOf(dataType))) {
            // 根据当前用户预案角色、未执行动作 (dataType = 2 查询所有)
            status = "1";
        }
        if (null != roleModelList && roleModelList.size() == 0) {
            return new ArrayList<>();
        }
        return contingencyInstanceInfoMapper.selectTaskActionList(type, status, roleModelList, batchNo);
    }

    @Override
    public List<ContingencyPlanInstanceVO> selectDisposalActionList(String batchNo, List<RoleModel> roleModelList, int dataType) {
        List<String> roles = new ArrayList<>();
        String type = "OPERATE";
        String status = "";
        if ("1".equals(String.valueOf(dataType))) {
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
            // 根据当前用户预案角色、未执行动作
            status = "1";
        } else if ("2".equals(String.valueOf(dataType))) {
            // 当前用户预案角色、全部动作数据
            for (RoleModel roleModel : roleModelList) {
                roles.add(roleModel.getRoleName());
            }
        }
        return contingencyInstanceInfoMapper.selectDisposalActionList(type, status, roles, batchNo);
    }

    @Override
    public void sendMessage(String roleCode, MessageVO messageVO) {
        //  添加极光推送开关
        if (auroraPushSwitch) {
            List<String> userIds = getUserIds(roleCode);
            if (!CollectionUtils.isEmpty(userIds)) {
                messageVO.setRecivers(userIds);
                sendMessage(messageVO);
            }
        }
    }

    private void sendMessage(MessageVO messageVO) {
        try {
            MessageModel model = new MessageModel();
            model.setMsgType(messageVO.getMsgType());
            // 关联id
            if (!ObjectUtils.isEmpty(messageVO.getRelationId())) {
                model.setRelationId(messageVO.getRelationId());
            }
            // 发送人ids
            model.setRecivers(messageVO.getRecivers());
            model.setIsSendApp(messageVO.getIsSendApp());
            model.setIsSendWeb(messageVO.getIsSendWeb());
            if (!ObjectUtils.isEmpty(messageVO) && !ObjectUtils.isEmpty(messageVO.getTitle())) {
                model.setTitle(messageVO.getTitle());
            }
            if (!ObjectUtils.isEmpty(messageVO) && !ObjectUtils.isEmpty(messageVO.getBody())) {
                model.setBody(messageVO.getBody());
            }
            model.setTerminal(messageVO.getTerminal());

            model.setCategory(messageVO.getCategory());

            if (StringUtils.isBlank(model.getBody()) || StringUtils.isBlank(model.getTitle())) {
                return;
            }
            model.setRelationId(String.valueOf(System.currentTimeMillis()));
            Systemctl.messageClient.create(model);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public boolean updateStatusById(String id, Boolean runStatus) {
        if (runStatus) {
            // 按钮去除
            ContingencyPlanInstance instance = contingencyPlanInstanceMapper.getMessageById(id);
            ContingencyPlanInstance contingencyPlanInstance = deleteButton(instance);
            return contingencyPlanInstanceMapper.updateStatusById(id, contingencyPlanInstance.getContent(), runStatus);
        } else {
            return contingencyPlanInstanceMapper.updateStatusById(id, null, runStatus);
        }
    }

    @Override
    public boolean updateTaskStatusById(String id, Boolean runStatus, String userName, String userId, String roleNames) {
        long start = System.currentTimeMillis();
        boolean bool = false;
        ResponseModel responseModel = null;
        try {
            responseModel = jcsFeign.getPersonImage(userId);
        } catch (Exception e) {
            e.printStackTrace();
        }
        long end = System.currentTimeMillis();
        System.out.println("调用jcs查询用户图像用时：" + (end - start) + "ms");
        String img = "";
        if (responseModel != null && "SUCCESS".equals(responseModel.getDevMessage())) {
            String resStr = JSON.toJSONString(responseModel.getResult()).replace("\"","");
            img = "null".equalsIgnoreCase(resStr) ? null : img;
        }
        img = StringUtils.isEmpty(img) ? personImg : img;
        if (runStatus) {
            ContingencyPlanInstance instance = contingencyPlanInstanceMapper.getMessageById(id);
            ContingencyPlanInstance contingencyPlanInstance = deleteButton(instance);
            bool = contingencyPlanInstanceMapper.updateTaskStatusById(id, contingencyPlanInstance.getContent(), runStatus, userName, userId, img, PlanReplyMessageEnum.TEXT.getCode(), roleNames);
        } else {
            bool = contingencyPlanInstanceMapper.updateTaskStatusById(id, null, runStatus, userName, userId, img, PlanReplyMessageEnum.TEXT.getCode(), roleNames);
        }

        if (bool) {
            if (redisTemplate.hasKey("planTask")) {
                redisTemplate.delete("planTask");
                String planTask = "";
                ContingencyPlanInstance instance = contingencyPlanInstanceMapper.getMessageById(id);
                List<ContingencyPlanInstanceVO> list = this.getTaskActionList(instance.getBatchNo(), 1);
                if (0 < list.size()) {
                    planTask = JSON.toJSONString(list);
                }
                JSONArray taskObjects = JSON.parseArray(planTask);
                redisTemplate.opsForValue().set("planTask", taskObjects);
            }
        }
        return bool;
    }

    @Override
    public ContingencyPlanInstance updateStatusByIdWeb(ContingencyPlanInstance contingencyPlanInstance, Boolean runStatus) {
        if (runStatus) {
            deleteButton(contingencyPlanInstance);
        }
        return contingencyPlanInstance;
    }

    private ContingencyPlanInstance deleteButton(ContingencyPlanInstance contingencyPlanInstance) {
        // 按钮去除
        String content = null;
        ButtonJsonVO buttonJsonVO;
        if (!ObjectUtils.isEmpty(contingencyPlanInstance.getContent())) {
            buttonJsonVO = JSONObject.parseObject(contingencyPlanInstance.getContent(), ButtonJsonVO.class);
            if (!ObjectUtils.isEmpty(buttonJsonVO.getOperate()) && buttonJsonVO.getOperate().size() > 1) {
                ArrayList<Map<String, Object>> maps = new ArrayList<>();
                // 取最后一个按钮
                maps.add(buttonJsonVO.getOperate().get(buttonJsonVO.getOperate().size() - 1));
                buttonJsonVO.setOperate(maps);
                content = JSONObject.toJSONString(buttonJsonVO);
                contingencyPlanInstance.setContent(content);
            }
        }
        return contingencyPlanInstance;
    }

    private List<String> getUserIds(String roleCode) {

        return planOperationRecordMapper.getArraysUserIds(roleCode);
    }

    public List<String> getUserName(String roleCode) {

        return planOperationRecordMapper.getArraysUserName(roleCode);
    }

    @Override
    public String getLastBatchNo() {
        return planOperationRecordMapper.getLastBatchNo();
    }

    public int getLastPlanPattern() {
        return planOperationRecordMapper.getLastPlanPattern();
    }
}
